@@ -63,6 +63,12 @@ def create_client_runner_args(project_name):
     parser.add_argument("--db_server_password", type=str, default=None)
     parser.add_argument("--db_server_port", type=int, default=6379)
     parser.add_argument("--cpuset_start_pos", type=int, default=0)
+    parser.add_argument(
+        "--maxmemory",
+        type=int,
+        default=0,
+        help="If specified will not retrieved the maxmemory from the DB Server and will use this limit. If 0 will read the this value from the DB servers.",
+    )
     parser.add_argument(
         "--tests-priority-lower-limit",
         type=int,
@@ -184,10 +190,22 @@ def create_client_runner_args(project_name):
         type=int,
         help="override memtier test-time for each benchmark. By default will preserve test time specified in test spec",
     )
+    parser.add_argument(
+        "--benchmark_local_install",
+        default=False,
+        action="store_true",
+        help="Assume benchmarking tool (e.g. memtier benchmark) is installed locally and execute it without using a docker container.",
+    )
     parser.add_argument(
         "--override-test-runs",
         default=1,
         type=int,
         help="override memtier number of runs for each benchmark. By default will run once each test",
     )
+    parser.add_argument(
+        "--cluster-mode",
+        default=False,
+        action="store_true",
+        help="Run client in cluster mode.",
+    )
     return parser
