@@ -220,6 +220,7 @@ def prepare_memtier_benchmark_parameters(
     override_memtier_test_time=0,
     override_test_runs=1,
 ):
+    arbitrary_command = False
     benchmark_command = [
         full_benchmark_path,
         "--port",
@@ -256,7 +257,9 @@ def prepare_memtier_benchmark_parameters(
     benchmark_command_str = " ".join(benchmark_command)
     if "arguments" in clientconfig:
         benchmark_command_str = benchmark_command_str + " " + clientconfig["arguments"]
-    logging.info(override_memtier_test_time)
+
+    if "--command" in benchmark_command_str:
+        arbitrary_command = True
 
     if override_test_runs > 1:
         benchmark_command_str = re.sub(
@@ -330,7 +333,7 @@ def prepare_memtier_benchmark_parameters(
             benchmark_command_str,
         )
 
-    return None, benchmark_command_str
+    return None, benchmark_command_str, arbitrary_command
 
 
 def process_self_contained_coordinator_stream(
@@ -410,7 +413,7 @@ def process_self_contained_coordinator_stream(
                     port = args.db_server_port
                     host = args.db_server_host
                     password = args.db_server_password
-
+                    oss_cluster_api_enabled = args.cluster_mode
                     ssl_cert_reqs = "required"
                     if tls_skip_verify:
                         ssl_cert_reqs = None
@@ -425,13 +428,41 @@ def process_self_contained_coordinator_stream(
                         ssl_ca_certs=tls_cacert,
                         ssl_check_hostname=False,
                     )
+                    setup_name = "oss-standalone"
                     r.ping()
                     redis_conns = [r]
+                    if oss_cluster_api_enabled:
+                        redis_conns = []
+                        logging.info("updating redis connections from cluster slots")
+                        slots = r.cluster("slots")
+                        for slot in slots:
+                            # Master for slot range represented as nested networking information starts at pos 2
+                            # example: [0, 5460, [b'127.0.0.1', 30001, b'eccd21c2e7e9b7820434080d2e394cb8f2a7eff2', []]]
+                            slot_network_info = slot[2]
+                            prefered_endpoint = slot_network_info[0]
+                            prefered_port = slot_network_info[1]
+                            shard_conn = redis.StrictRedis(
+                                host=prefered_endpoint,
+                                port=prefered_port,
+                                password=password,
+                                ssl=tls_enabled,
+                                ssl_cert_reqs=ssl_cert_reqs,
+                                ssl_keyfile=tls_key,
+                                ssl_certfile=tls_cert,
+                                ssl_ca_certs=tls_cacert,
+                                ssl_check_hostname=False,
+                            )
+                            redis_conns.append(shard_conn)
+                        logging.info(
+                            "There are a total of {} shards".format(len(redis_conns))
+                        )
+                        setup_name = "oss-cluster"
+
                     redis_pids = []
-                    first_redis_pid = r.info()["process_id"]
-                    redis_pids.append(first_redis_pid)
+                    for conn in redis_conns:
+                        redis_pid = conn.info()["process_id"]
+                        redis_pids.append(redis_pid)
 
-                    setup_name = "oss-standalone"
                     github_actor = f"{tf_triggering_env}-{running_platform}"
                     dso = "redis-server"
                     profilers_artifacts_matrix = []
@@ -457,38 +488,20 @@ def process_self_contained_coordinator_stream(
                     )
                     if args.flushall_on_every_test_start:
                         logging.info("Sending FLUSHALL to the DB")
-                        r.flushall()
+                        for conn in redis_conns:
+                            conn.flushall()
 
-                    benchmark_required_memory = 0
+                    benchmark_required_memory = get_benchmark_required_memory(
+                        benchmark_config
+                    )
                     maxmemory = 0
-                    if "resources" in benchmark_config["dbconfig"]:
-                        resources = benchmark_config["dbconfig"]["resources"]
-                        if "requests" in resources:
-                            resources_requests = benchmark_config["dbconfig"][
-                                "resources"
-                            ]["requests"]
-                            if "memory" in resources_requests:
-                                benchmark_required_memory = resources_requests["memory"]
-                                benchmark_required_memory = parse_size(
-                                    benchmark_required_memory
-                                )
-                                logging.info(
-                                    "Benchmark required memory: {} Bytes".format(
-                                        benchmark_required_memory
-                                    )
-                                )
-
-                    maxmemory = r.info("memory")["maxmemory"]
-                    if maxmemory == 0:
-                        total_system_memory = r.info("memory")["total_system_memory"]
-                        logging.info(
-                            " Using total system memory as max {}".format(
-                                total_system_memory
-                            )
-                        )
-                        maxmemory = total_system_memory
+                    if args.maxmemory > 0:
+                        maxmemory = args.maxmemory
+                    else:
+                        for conn in redis_conns:
+                            maxmemory = maxmemory + get_maxmemory(conn)
                     if benchmark_required_memory > maxmemory:
-                        logging.WARN(
+                        logging.warning(
                             "Skipping test {} given maxmemory of server is bellow the benchmark required memory: {} < {}".format(
                                 test_name, maxmemory, benchmark_required_memory
                             )
@@ -560,7 +573,7 @@ def process_self_contained_coordinator_stream(
                         continue
 
                     if "preload_tool" in benchmark_config["dbconfig"]:
-                        data_prepopulation_step(
+                        res = data_prepopulation_step(
                             benchmark_config,
                             benchmark_tool_workdir,
                             client_cpuset_cpus,
@@ -576,19 +589,30 @@ def process_self_contained_coordinator_stream(
                             test_tls_key,
                             test_tls_cacert,
                             resp_version,
+                            args.benchmark_local_install,
+                            password,
+                            oss_cluster_api_enabled,
                         )
+                        if res is False:
+                            logging.warning(
+                                "Skipping this test given preload result was false"
+                            )
+                            continue
                     execute_init_commands(
                         benchmark_config, r, dbconfig_keyname="dbconfig"
                     )
 
                     used_memory_check(
-                        test_name, benchmark_required_memory, r, "start of benchmark"
+                        test_name,
+                        benchmark_required_memory,
+                        redis_conns,
+                        "start of benchmark",
                     )
 
                     logging.info("Checking if there is a keyspace check being enforced")
                     dbconfig_keyspacelen_check(
                         benchmark_config,
-                        [r],
+                        redis_conns,
                     )
 
                     if dry_run_include_preload is True:
@@ -619,6 +643,8 @@ def process_self_contained_coordinator_stream(
                             local_benchmark_output_filename
                         )
                     )
+                    arbitrary_command = False
+
                     if "memtier_benchmark" not in benchmark_tool:
                         # prepare the benchmark command
                         (
@@ -638,6 +664,7 @@ def process_self_contained_coordinator_stream(
                         (
                             _,
                             benchmark_command_str,
+                            arbitrary_command,
                         ) = prepare_memtier_benchmark_parameters(
                             benchmark_config["clientconfig"],
                             full_benchmark_path,
@@ -645,7 +672,7 @@ def process_self_contained_coordinator_stream(
                             host,
                             password,
                             local_benchmark_output_filename,
-                            False,
+                            oss_cluster_api_enabled,
                             tls_enabled,
                             tls_skip_verify,
                             test_tls_cert,
@@ -656,6 +683,16 @@ def process_self_contained_coordinator_stream(
                             override_test_runs,
                         )
 
+                    if (
+                        arbitrary_command
+                        and oss_cluster_api_enabled
+                        and "memtier" in benchmark_tool
+                    ):
+                        logging.warning(
+                            "Forcing skip this test given there is an arbitrary commmand and memtier usage. Check https://github.com/RedisLabs/memtier_benchmark/pull/117 ."
+                        )
+                        continue
+
                     client_container_image = extract_client_container_image(
                         benchmark_config
                     )
@@ -673,32 +710,54 @@ def process_self_contained_coordinator_stream(
                         profiler_frequency,
                         profiler_call_graph_mode,
                     )
-                    logging.info(
-                        "Using docker image {} as benchmark client image (cpuset={}) with the following args: {}".format(
-                            client_container_image,
-                            client_cpuset_cpus,
-                            benchmark_command_str,
-                        )
-                    )
+
                     # run the benchmark
                     benchmark_start_time = datetime.datetime.now()
 
-                    client_container_stdout = docker_client.containers.run(
-                        image=client_container_image,
-                        volumes={
-                            temporary_dir_client: {
-                                "bind": client_mnt_point,
-                                "mode": "rw",
+                    if args.benchmark_local_install:
+                        logging.info("Running memtier benchmark outside of docker")
+                        benchmark_command_str = (
+                            "taskset -c "
+                            + client_cpuset_cpus
+                            + " "
+                            + benchmark_command_str
+                        )
+                        logging.info(
+                            "Running memtier benchmark command {}".format(
+                                benchmark_command_str
+                            )
+                        )
+                        stream = os.popen(benchmark_command_str)
+                        client_container_stdout = stream.read()
+                        move_command = "mv {} {}".format(
+                            local_benchmark_output_filename, temporary_dir_client
+                        )
+                        os.system(move_command)
+                    else:
+                        logging.info(
+                            "Using docker image {} as benchmark client image (cpuset={}) with the following args: {}".format(
+                                client_container_image,
+                                client_cpuset_cpus,
+                                benchmark_command_str,
+                            )
+                        )
+
+                        client_container_stdout = docker_client.containers.run(
+                            image=client_container_image,
+                            volumes={
+                                temporary_dir_client: {
+                                    "bind": client_mnt_point,
+                                    "mode": "rw",
+                                },
                             },
-                        },
-                        auto_remove=True,
-                        privileged=True,
-                        working_dir=benchmark_tool_workdir,
-                        command=benchmark_command_str,
-                        network_mode="host",
-                        detach=False,
-                        cpuset_cpus=client_cpuset_cpus,
-                    )
+                            auto_remove=True,
+                            privileged=True,
+                            working_dir=benchmark_tool_workdir,
+                            command=benchmark_command_str,
+                            network_mode="host",
+                            detach=False,
+                            cpuset_cpus=client_cpuset_cpus,
+                        )
 
                     benchmark_end_time = datetime.datetime.now()
                     benchmark_duration_seconds = (
@@ -725,12 +784,16 @@ def process_self_contained_coordinator_stream(
                     logging.info("Printing client tool stdout output")
 
                     used_memory_check(
-                        test_name, benchmark_required_memory, r, "end of benchmark"
+                        test_name,
+                        benchmark_required_memory,
+                        redis_conns,
+                        "end of benchmark",
                     )
 
                     if args.flushall_on_every_test_end:
                         logging.info("Sending FLUSHALL to the DB")
-                        r.flushall()
+                        for r in redis_conns:
+                            r.flushall()
                     datapoint_time_ms = start_time_ms
 
                     post_process_benchmark_results(
@@ -878,8 +941,39 @@ def process_self_contained_coordinator_stream(
         )
 
 
-def used_memory_check(test_name, benchmark_required_memory, r, stage):
-    used_memory = r.info("memory")["used_memory"]
+def get_maxmemory(r):
+    maxmemory = int(r.info("memory")["maxmemory"])
+    if maxmemory == 0:
+        total_system_memory = int(r.info("memory")["total_system_memory"])
+        logging.info(" Using total system memory as max {}".format(total_system_memory))
+        maxmemory = total_system_memory
+    else:
+        logging.info(" Detected redis maxmemory config value {}".format(maxmemory))
+
+    return maxmemory
+
+
+def get_benchmark_required_memory(benchmark_config):
+    benchmark_required_memory = 0
+    if "resources" in benchmark_config["dbconfig"]:
+        resources = benchmark_config["dbconfig"]["resources"]
+        if "requests" in resources:
+            resources_requests = benchmark_config["dbconfig"]["resources"]["requests"]
+            if "memory" in resources_requests:
+                benchmark_required_memory = resources_requests["memory"]
+                benchmark_required_memory = int(parse_size(benchmark_required_memory))
+                logging.info(
+                    "Benchmark required memory: {} Bytes".format(
+                        benchmark_required_memory
+                    )
+                )
+    return benchmark_required_memory
+
+
+def used_memory_check(test_name, benchmark_required_memory, redis_conns, stage):
+    used_memory = 0
+    for conn in redis_conns:
+        used_memory = used_memory + conn.info("memory")["used_memory"]
     used_memory_gb = int(math.ceil(float(used_memory) / 1024.0 / 1024.0 / 1024.0))
     logging.info("Benchmark used memory at {}: {}g".format(stage, used_memory_gb))
     if used_memory > benchmark_required_memory:
@@ -963,8 +1057,11 @@ def data_prepopulation_step(
     tls_key=None,
     tls_cacert=None,
     resp_version=None,
+    benchmark_local_install=False,
     password=None,
+    oss_cluster_api_enabled=False,
 ):
+    result = True
     # setup the benchmark
     (
         start_time,
@@ -986,14 +1083,18 @@ def data_prepopulation_step(
 
     if "memtier_benchmark" in preload_tool:
         override_memtier_test_time_preload = 0
-        (_, preload_command_str,) = prepare_memtier_benchmark_parameters(
+        (
+            _,
+            preload_command_str,
+            arbitrary_command,
+        ) = prepare_memtier_benchmark_parameters(
             benchmark_config["dbconfig"]["preload_tool"],
             full_benchmark_path,
             port,
             host,
             password,
             local_benchmark_output_filename,
-            False,
+            oss_cluster_api_enabled,
             tls_enabled,
             tls_skip_verify,
             tls_cert,
@@ -1002,33 +1103,59 @@ def data_prepopulation_step(
             resp_version,
             override_memtier_test_time_preload,
         )
-
-        logging.info(
-            "Using docker image {} as benchmark PRELOAD image (cpuset={}) with the following args: {}".format(
-                preload_image,
-                client_cpuset_cpus,
-                preload_command_str,
+        if arbitrary_command is True and oss_cluster_api_enabled:
+            logging.warning(
+                "Skipping this test given it implies arbitrary command on an cluster setup. Not supported on memtier: https://github.com/RedisLabs/memtier_benchmark/pull/117"
             )
-        )
+            result = False
+            return result
+
         # run the benchmark
         preload_start_time = datetime.datetime.now()
 
-        client_container_stdout = docker_client.containers.run(
-            image=preload_image,
-            volumes={
-                temporary_dir: {
-                    "bind": client_mnt_point,
-                    "mode": "rw",
+        if benchmark_local_install:
+            logging.info("Running memtier benchmark outside of docker")
+
+            preload_command_str = (
+                "taskset -c " + client_cpuset_cpus + " " + preload_command_str
+            )
+            logging.info(
+                "Pre-loading using memtier benchmark command {}".format(
+                    preload_command_str
+                )
+            )
+            stream = os.popen(preload_command_str)
+            client_container_stdout = stream.read()
+
+            move_command = "mv {} {}".format(
+                local_benchmark_output_filename, temporary_dir
+            )
+            os.system(move_command)
+
+        else:
+            logging.info(
+                "Using docker image {} as benchmark PRELOAD image (cpuset={}) with the following args: {}".format(
+                    preload_image,
+                    client_cpuset_cpus,
+                    preload_command_str,
+                )
+            )
+            client_container_stdout = docker_client.containers.run(
+                image=preload_image,
+                volumes={
+                    temporary_dir: {
+                        "bind": client_mnt_point,
+                        "mode": "rw",
+                    },
                 },
-            },
-            auto_remove=True,
-            privileged=True,
-            working_dir=benchmark_tool_workdir,
-            command=preload_command_str,
-            network_mode="host",
-            detach=False,
-            cpuset_cpus=client_cpuset_cpus,
-        )
+                auto_remove=True,
+                privileged=True,
+                working_dir=benchmark_tool_workdir,
+                command=preload_command_str,
+                network_mode="host",
+                detach=False,
+                cpuset_cpus=client_cpuset_cpus,
+            )
 
         preload_end_time = datetime.datetime.now()
         preload_duration_seconds = calculate_client_tool_duration_and_check(
@@ -1040,6 +1167,7 @@ def data_prepopulation_step(
                 client_container_stdout,
             )
         )
+    return result
 
 
 def generate_cpuset_cpus(ceil_db_cpu_limit, current_cpu_pos):
